/*
  UtLcdEdit.ino
  Unit tests.

  release : 0.0.0 - 7 April 2011

  ------------------------------------------------------------------------------
  Copyright (c) 2011 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include <Arduino.h>
#include <LiquidCrystal.h>
#include "ArduinoUnit.h"
#include "LcdUi.h"

TestSuite suite;

PROGMEM prog_char DEFC[] = "cac";    // character
PROGMEM prog_char DEFC2[] = "cad";   // character
PROGMEM prog_char DEFB[] = "b1133";  // byte
PROGMEM prog_char DEFI[] = "i1133";  // integer
PROGMEM prog_char DEFS[] = "saacc";  // string

void setup() {}

test(edit_char_P_NONE) {
  char value[] = " ";
  LCDKEY key = LCDKEY_NONE;
  key = LcdEditChar_P(DEFC,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("a",value);
}

test(edit_char_P_nop) {
  char value[] = "b";
  LCDKEY key = LCDKEY_SELECT;
  key = LcdEditChar_P(DEFC,key,value);
  assertEquals(LCDKEY_SELECT,key);
  key = LCDKEY_ESC;
  key = LcdEditChar_P(DEFC,key,value);
  assertEquals(LCDKEY_ESC,key);
  key = LCDKEY_LEFT;
  key = LcdEditChar_P(DEFC,key,value);
  assertEquals(LCDKEY_LEFT,key);
  key = LCDKEY_RIGHT;
  key = LcdEditChar_P(DEFC,key,value);
  assertEquals(LCDKEY_RIGHT,key);
  assertEquals("b",value);
}

test(edit_char_P_UP_1) {
  char value[] = "b";
  LCDKEY key = LCDKEY_UP;
  key = LcdEditChar_P(DEFC2,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("c",value);
}

test(edit_char_P_UP_round) {
  char value[] = "c";
  LCDKEY key = LCDKEY_UP;
  key = LcdEditChar_P(DEFC,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("c",value);
}

test(edit_char_P_DOWN_1) {
  char value[] = "b";
  LCDKEY key = LCDKEY_DOWN;
  key = LcdEditChar_P(DEFC,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("a",value);
}

test(edit_char_P_DOWN_round) {
  char value[] = "a";
  LCDKEY key = LCDKEY_DOWN;
  key = LcdEditChar_P(DEFC,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("a",value);
}

test(edit_byte_NONE) {
  char value[] = "00";
  char x = 0;
  LCDKEY key = LCDKEY_NONE;
  key = LcdEditByte(2,11,33,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("11",value);
}

test(edit_byte_nop) {
  char value[] = "22";
  char x = 0;
  LCDKEY key = LCDKEY_SELECT;
  key = LcdEditByte(2,11,33,&x,key,value);
  assertEquals(LCDKEY_SELECT,key);
  key = LCDKEY_ESC;
  key = LcdEditByte(2,11,33,&x,key,value);
  assertEquals(LCDKEY_ESC,key);
  assertEquals("22",value);
}

test(edit_byte_LEFT) {
  char value[] = "22";
  char x = 1;
  LCDKEY key = LCDKEY_LEFT;
  key = LcdEditByte(2,11,33,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals(0,x);
  key = LCDKEY_LEFT;
  key = LcdEditByte(2,11,33,&x,key,value);
  assertEquals(LCDKEY_LEFT,key);
  assertEquals(0,x);
}

test(edit_byte_RIGHT) {
  char value[] = "22";
  char x = 0;
  LCDKEY key = LCDKEY_RIGHT;
  key = LcdEditByte(2,11,33,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals(1,x);
  key = LCDKEY_RIGHT;
  key = LcdEditByte(2,11,33,&x,key,value);
  assertEquals(LCDKEY_RIGHT,key);
  assertEquals(1,x);
}

test(edit_byte_UP_1) {
  char value[] = "22";
  char x = 0;
  LCDKEY key = LCDKEY_UP;
  key = LcdEditByte(2,11,33,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("32",value);
}

test(edit_byte_UP_round) {
  char value[] = "32";
  char x = 0;
  LCDKEY key = LCDKEY_UP;
  key = LcdEditByte(2,11,33,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("32",value);
}

test(edit_byte_DOWN_1) {
  char value[] = "22";
  char x = 0;
  LCDKEY key = LCDKEY_DOWN;
  key = LcdEditByte(2,11,33,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("12",value);
}

test(edit_byte_DOWN_2) {
  char value[] = "21";
  char x = 1;
  LCDKEY key = LCDKEY_DOWN;
  key = LcdEditByte(2,11,33,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("20",value);
}

test(edit_byte_DOWN_round) {
  char value[] = "12";
  char x = 0;
  LCDKEY key = LCDKEY_DOWN;
  key = LcdEditByte(2,11,33,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("12",value);
}

test(edit_byte_P_NONE) {
  char value[] = "00";
  char x = 0;
  LCDKEY key = LCDKEY_NONE;
  key = LcdEditByte_P(DEFB,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("11",value);
}

test(edit_byte_P_nop) {
  char value[] = "22";
  char x = 0;
  LCDKEY key = LCDKEY_SELECT;
  key = LcdEditByte_P(DEFB,2,&x,key,value);
  assertEquals(LCDKEY_SELECT,key);
  key = LCDKEY_ESC;
  key = LcdEditByte_P(DEFB,2,&x,key,value);
  assertEquals(LCDKEY_ESC,key);
  assertEquals("22",value);
}

test(edit_byte_P_LEFT) {
  char value[] = "22";
  char x = 1;
  LCDKEY key = LCDKEY_LEFT;
  key = LcdEditByte_P(DEFB,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals(0,x);
  key = LCDKEY_LEFT;
  key = LcdEditByte_P(DEFB,2,&x,key,value);
  assertEquals(LCDKEY_LEFT,key);
  assertEquals(0,x);
}

test(edit_byte_P_RIGHT) {
  char value[] = "22";
  char x = 0;
  LCDKEY key = LCDKEY_RIGHT;
  key = LcdEditByte_P(DEFB,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals(1,x);
  key = LCDKEY_RIGHT;
  key = LcdEditByte_P(DEFB,2,&x,key,value);
  assertEquals(LCDKEY_RIGHT,key);
  assertEquals(1,x);
}

test(edit_byte_P_UP_1) {
  char value[] = "22";
  char x = 0;
  LCDKEY key = LCDKEY_UP;
  key = LcdEditByte_P(DEFB,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("32",value);
}

test(edit_byte_P_UP_round) {
  char value[] = "32";
  char x = 0;
  LCDKEY key = LCDKEY_UP;
  key = LcdEditByte_P(DEFB,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("32",value);
}

test(edit_byte_P_DOWN_1) {
  char value[] = "22";
  char x = 0;
  LCDKEY key = LCDKEY_DOWN;
  key = LcdEditByte_P(DEFB,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("12",value);
}

test(edit_byte_P_DOWN_2) {
  char value[] = "21";
  char x = 1;
  LCDKEY key = LCDKEY_DOWN;
  key = LcdEditByte_P(DEFB,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("20",value);
}

test(edit_byte_P_DOWN_round) {
  char value[] = "12";
  char x = 0;
  LCDKEY key =LCDKEY_DOWN;
  key = LcdEditByte_P(DEFB,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("12",value);
}

test(edit_int_P_NONE) {
  char value[] = "00";
  char x = 0;
  LCDKEY key = LCDKEY_NONE;
  key = LcdEditInt_P(DEFI,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("11",value);
}

test(edit_int_P_nop) {
  char value[] = "22";
  char x = 0;
  LCDKEY key = LCDKEY_SELECT;
  key = LcdEditInt_P(DEFI,2,&x,key,value);
  assertEquals(LCDKEY_SELECT,key);
  key = LCDKEY_ESC;
  key = LcdEditInt_P(DEFI,2,&x,key,value);
  assertEquals(LCDKEY_ESC,key);
  assertEquals("22",value);
}

test(edit_int_P_LEFT) {
  char value[] = "22";
  char x = 1;
  LCDKEY key = LCDKEY_LEFT;
  key = LcdEditInt_P(DEFI,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals(0,x);
  key = LCDKEY_LEFT;
  key = LcdEditInt_P(DEFI,2,&x,key,value);
  assertEquals(LCDKEY_LEFT,key);
  assertEquals(0,x);
}

test(edit_int_P_RIGHT) {
  char value[] = "22";
  char x = 0;
  LCDKEY key = LCDKEY_RIGHT;
  key = LcdEditInt_P(DEFI,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals(1,x);
  key = LCDKEY_RIGHT;
  key = LcdEditInt_P(DEFI,2,&x,key,value);
  assertEquals(LCDKEY_RIGHT,key);
  assertEquals(1,x);
}

test(edit_int_P_UP_1) {
  char value[] = "22";
  char x = 0;
  LCDKEY key = LCDKEY_UP;
  key = LcdEditInt_P(DEFI,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("32",value);
}

test(edit_int_P_UP_round) {
  char value[] = "32";
  char x = 0;
  LCDKEY key = LCDKEY_UP;
  key = LcdEditInt_P(DEFI,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("32",value);
}

test(edit_int_P_DOWN_1) {
  char value[] = "22";
  char x = 0;
  LCDKEY key = LCDKEY_DOWN;
  key = LcdEditInt_P(DEFI,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("12",value);
}

test(edit_int_P_DOWN_2) {
  char value[] = "21";
  char x = 1;
  LCDKEY key = LCDKEY_DOWN;
  key = LcdEditInt_P(DEFI,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("20",value);
}

test(edit_int_P_DOWN_round) {
  char value[] = "12";
  char x = 0;
  LCDKEY key =LCDKEY_DOWN;
  key = LcdEditInt_P(DEFI,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("12",value);
}

test(edit_string_P_NONE) {
  char value[] = "00";
  char x = 0;
  LCDKEY key = LCDKEY_NONE;
  key = LcdEditString_P(DEFS,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("aa",value);
}

test(edit_string_P_nop) {
  char value[] = "bb";
  char x = 0;
  LCDKEY key = LCDKEY_SELECT;
  key = LcdEditString_P(DEFS,2,&x,key,value);
  assertEquals(LCDKEY_SELECT,key);
  key = LCDKEY_ESC;
  key = LcdEditString_P(DEFS,2,&x,key,value);
  assertEquals(LCDKEY_ESC,key);
  assertEquals("bb",value);
}

test(edit_string_P_LEFT) {
  char value[] = "bb";
  char x = 1;
  LCDKEY key = LCDKEY_LEFT;
  key = LcdEditString_P(DEFS,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals(0,x);
  key = LCDKEY_LEFT;
  key = LcdEditString_P(DEFS,2,&x,key,value);
  assertEquals(LCDKEY_LEFT,key);
  assertEquals(0,x);
}

test(edit_string_P_RIGHT) {
  char value[] = "bb";
  char x = 0;
  LCDKEY key = LCDKEY_RIGHT;
  key = LcdEditString_P(DEFS,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals(1,x);
  key = LCDKEY_RIGHT;
  key = LcdEditString_P(DEFS,2,&x,key,value);
  assertEquals(LCDKEY_RIGHT,key);
  assertEquals(1,x);
}

test(edit_string_P_UP_1) {
  char value[] = "bb";
  char x = 0;
  LCDKEY key = LCDKEY_UP;
  key = LcdEditString_P(DEFS,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("cb",value);
}

test(edit_string_P_UP_round) {
  char value[] = "cb";
  char x = 0;
  LCDKEY key = LCDKEY_UP;
  key = LcdEditString_P(DEFS,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("cb",value);
}

test(edit_string_P_DOWN_1) {
  char value[] = "bb";
  char x = 0;
  LCDKEY key = LCDKEY_DOWN;
  key = LcdEditString_P(DEFS,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("ab",value);
}

test(edit_string_P_DOWN_round) {
  char value[] = "ab";
  char x = 0;
  LCDKEY key = LCDKEY_DOWN;
  key = LcdEditString_P(DEFS,2,&x,key,value);
  assertEquals(LCDKEY_NONE,key);
  assertEquals("ab",value);
}

void loop() {
  suite.run();
}
