/*
  UTLcdEditTime.ino
  Unit tests.

  release : 0.0.0 - 7 April 2011

  ------------------------------------------------------------------------------
  Copyright (c) 2011 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include <Arduino.h>
#include <LiquidCrystal.h>
#include "ArduinoUnit.h"
#include "LcdUi.h"

TestSuite suite;

void setup() {}

test(edit_date_nop) {
  char value[] = "04/05/06";
  char x = 0;
  LCDKEY key = LCDKEY_NONE;
  LcdEditTimeDate(&x,key,value);
  assertEquals("04/05/06",value);
}

test(edit_time_nop) {
  char value[] = "01:02:03";
  char x = 0;
  LCDKEY key = LCDKEY_NONE;
  LcdEditTimeTime(&x,key,value);
  assertEquals("01:02:03",value);
}

test(edit_time_hour_ok) {
  char value[] = "00:00:00";
  char x = 0;                           // tens of hours
  LCDKEY key = LCDKEY_UP;                // increment to 10
  LcdEditTimeTime(&x,key,value);
  assertEquals("10:00:00",value);
  key = LCDKEY_DOWN;                     // decrement to 0
  LcdEditTimeTime(&x,key,value);
  key = LCDKEY_DOWN;
  LcdEditTimeTime(&x,key,value);
  assertEquals("00:00:00",value);
  key = LCDKEY_RIGHT;                    // units of hours
  LcdEditTimeTime(&x,key,value);
  key = LCDKEY_UP;                       // increment to 1
  LcdEditTimeTime(&x,key,value);
  assertEquals("01:00:00",value);
}

test(edit_time_minute_ok) {
  char value[] = "00:00:00";
  char x = 3;                           // tens of minutes
  LCDKEY key = LCDKEY_UP;                // increment to 10
  LcdEditTimeTime(&x,key,value);
  assertEquals("00:10:00",value);
  key = LCDKEY_DOWN;                     // decrement to 0
  LcdEditTimeTime(&x,key,value);
  key = LCDKEY_DOWN;
  LcdEditTimeTime(&x,key,value);
  assertEquals("00:00:00",value);
  key = LCDKEY_RIGHT;                    // units of minutes
  LcdEditTimeTime(&x,key,value);
  key = LCDKEY_UP;                       // increment to 1
  LcdEditTimeTime(&x,key,value);
  assertEquals("00:01:00",value);
}

test(edit_time_second_ok) {
  char value[] = "00:00:00";
  char x = 6;                           // tens of seconds
  LCDKEY key = LCDKEY_UP;                // increment to 10
  LcdEditTimeTime(&x,key,value);
  assertEquals("00:00:10",value);
  key = LCDKEY_DOWN;                     // decrement to 0
  LcdEditTimeTime(&x,key,value);
  key = LCDKEY_DOWN;
  LcdEditTimeTime(&x,key,value);
  assertEquals("00:00:00",value);
  key = LCDKEY_RIGHT;                    // units of seconds
  LcdEditTimeTime(&x,key,value);
  key = LCDKEY_UP;                       // increment to 1
  LcdEditTimeTime(&x,key,value);
  assertEquals("00:00:01",value);
}

test(edit_time_day_ok) {
  char value[] = "01/01/10";
  char x = 0;                           // tens of days
  LCDKEY key = LCDKEY_UP;                // increment to 10
  LcdEditTimeDate(&x,key,value);
  assertEquals("11/01/10",value);
  key = LCDKEY_DOWN;                     // decrement to 1
  LcdEditTimeDate(&x,key,value);
  key = LCDKEY_DOWN;
  LcdEditTimeDate(&x,key,value);
  assertEquals("01/01/10",value);
  key = LCDKEY_RIGHT;                    // units of days
  LcdEditTimeDate(&x,key,value);
  key = LCDKEY_UP;                       // increment to 2
  LcdEditTimeDate(&x,key,value);
  assertEquals("02/01/10",value);
}

test(edit_time_day_leap_ok) {
  char value[] = "28/02/11";
  char x = 1;                           // units of days
  LCDKEY key = LCDKEY_UP;                // increment to 28
  LcdEditTimeDate(&x,key,value);
  assertEquals("28/02/11",value);
  value[1] = '9';
  value[7] = '2';
  key = LCDKEY_UP;                       // increment to 29
  LcdEditTimeDate(&x,key,value);
  assertEquals("29/02/12",value);
}

test(edit_time_month_ok) {
  char value[] = "01/01/10";
  char x = 3;                           // tens of months
  LCDKEY key = LCDKEY_UP;                // increment to 11
  LcdEditTimeDate(&x,key,value);
  assertEquals("01/11/10",value);
  key = LCDKEY_DOWN;                     // decrement to 1
  LcdEditTimeDate(&x,key,value);
  assertEquals("01/01/10",value);
  key = LCDKEY_RIGHT;                    // units of months
  LcdEditTimeDate(&x,key,value);
  key = LCDKEY_UP;                       // increment to 2
  LcdEditTimeDate(&x,key,value);
  assertEquals("01/02/10",value);
}

test(edit_time_year_ok) {
  char value[] = "01/01/10";
  char x = 6;                           // tens of years
  LCDKEY key = LCDKEY_UP;                // increment to 20
  LcdEditTimeDate(&x,key,value);
  assertEquals("01/01/20",value);
  key = LCDKEY_DOWN;                     // decrement to 10
  LcdEditTimeDate(&x,key,value);
  assertEquals("01/01/10",value);
  key = LCDKEY_RIGHT;                    // units of years
  LcdEditTimeDate(&x,key,value);
  key = LCDKEY_UP;                       // increment to 11
  LcdEditTimeDate(&x,key,value);
  assertEquals("01/01/11",value);
}

void loop() {
  suite.run();
}
