/*
  Utkey = LcdField.ino
  Unit tests.

  release : 0.0.0 - 7 April 2011

  ------------------------------------------------------------------------------
  Copyright (c) 2011 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include <str_i.h>
#include <Arduino.h>
#include <LiquidCrystal.h>
#include "ArduinoUnit.h"
#include "LcdUi.h"

TestSuite suite;

char DEFC[] PROGMEM = "cac";        // character
char DEFB[] PROGMEM = "b1133";      // byte
char DEFI[] PROGMEM = "i4477";      // integer
char DEFS[] PROGMEM = "saacc";      // str_i
char DEFT[] PROGMEM = "t07:32:33";  // time
char DEFD[] PROGMEM = "d18:03:50";  // date
char DEFO[] PROGMEM = "o003";       // output

char str_i[16];
LcdField field;

void setup() {}

test(init_character) {
  memset(str_i,0,16);
  field.DEF = DEFC;
  field.data = str_i;
  LcdFieldInit(&field,true);
  assertEquals(1,field.length);
  assertEquals("a",field.data);
}

test(init_byte) {
  memset(str_i,0,16);
  field.DEF = DEFB;
  field.data = str_i;
  LcdFieldInit(&field,true);
  assertEquals(2,field.length);
  assertEquals("11",field.data);
}

test(init_integer) {
  memset(str_i,0,16);
  field.DEF = DEFI;
  field.data = str_i;
  LcdFieldInit(&field,true);
  assertEquals(2,field.length);
  assertEquals("44",field.data);
}

test(init_str_i) {
  memset(str_i,0,16);
  field.DEF = DEFS;
  field.data = str_i;
  LcdFieldInit(&field,true);
  assertEquals(2,field.length);
  assertEquals("aa",field.data);
}

test(init_time) {
  memset(str_i,0,16);
  field.DEF = DEFT;
  field.data = str_i;
  LcdFieldInit(&field,true);
  assertEquals(8,field.length);
  assertEquals("07:32:33",field.data);
}

test(init_date) {
  memset(str_i,0,16);
  field.DEF = DEFD;
  field.data = str_i;
  LcdFieldInit(&field,true);
  assertEquals(8,field.length);
  assertEquals("18:03:50",field.data);
}

test(init_output) {
  memset(str_i,0,16);
  field.DEF = DEFO;
  field.data = str_i;
  LcdFieldInit(&field,true);
  assertEquals(3,field.length);
  assertEquals("---",field.data);
}

test(edit_char) {
  memset(str_i,0,16);
  field.DEF = DEFC;
  field.data = str_i;
  LcdFieldInit(&field,true);
  LCDKEY key = LCDKEY_SELECT;
  LcdFieldEdit(key,&field);
  key = LCDKEY_UP;
  LcdFieldEdit(key,&field);
  assertEquals("b",field.data);
}

test(edit_byte) {
  memset(str_i,0,16);
  field.DEF = DEFB;
  field.data = str_i;
  LcdFieldInit(&field,true);
  LCDKEY key = LCDKEY_SELECT;
  LcdFieldEdit(key,&field);
  key = LCDKEY_UP;
  LcdFieldEdit(key,&field);
  assertEquals("21",field.data);
}

test(edit_int) {
  memset(str_i,0,16);
  field.DEF = DEFI;
  field.data = str_i;
  LcdFieldInit(&field,true);
  LCDKEY key = LCDKEY_SELECT;
  LcdFieldEdit(key,&field);
  key = LCDKEY_UP;
  LcdFieldEdit(key,&field);
  assertEquals("54",field.data);
}

test(edit_str_i) {
  memset(str_i,0,16);
  field.DEF = DEFS;
  field.data = str_i;
  LcdFieldInit(&field,true);
  LCDKEY key = LCDKEY_SELECT;
  LcdFieldEdit(key,&field);
  key = LCDKEY_UP;
  LcdFieldEdit(key,&field);
  assertEquals("ba",field.data);
}

test(edit_time) {
  memset(str_i,0,16);
  field.DEF = DEFT;
  field.data = str_i;
  LcdFieldInit(&field,true);
  LCDKEY key = LCDKEY_SELECT;
  LcdFieldEdit(key,&field);
  key = LCDKEY_UP;
  LcdFieldEdit(key,&field);
  assertEquals("17:32:33",field.data);
}

test(edit_date) {
  memset(str_i,0,16);
  field.DEF = DEFD;
  field.data = str_i;
  LcdFieldInit(&field,true);
  LCDKEY key = LCDKEY_SELECT;
  LcdFieldEdit(key,&field);
  key = LCDKEY_UP;
  LcdFieldEdit(key,&field);
  assertEquals("28/03/50",field.data);
}

test(edit_not) {
  memset(str_i,0,16);
  field.DEF = DEFC;
  field.data = str_i;
  LcdFieldInit(&field,true);
  LCDKEY key = LCDKEY_UP;
  assertEquals(LCDKEY_UP,LcdFieldEdit(key,&field));
}

test(status_1) { //long test cases with different asserts don't seem to work...
  memset(str_i,0,16);
  field.DEF = DEFC;
  field.data = str_i;
  LcdFieldInit(&field,true);
  LCDKEY key = LCDKEY_SELECT;
  assertEquals(LCDKEY_NONE,LcdFieldEdit(key,&field));
  assertEquals(LCDEDIT_EDIT,field.status);
  key = LCDKEY_UP;
  assertEquals(LCDKEY_NONE,LcdFieldEdit(key,&field));
  assertEquals(LCDEDIT_CHANGED,field.status);
}

test(status_2) {//part 2 of status test
  memset(str_i,0,16);
  field.DEF = DEFC;
  field.data = str_i;
  LcdFieldInit(&field,true);
  LCDKEY key = LCDKEY_SELECT;
  LcdFieldEdit(key,&field);
  key = LCDKEY_UP;
  LcdFieldEdit(key,&field);
  key = LCDKEY_ESC;
  assertEquals(LCDKEY_NONE,LcdFieldEdit(key,&field));
  assertEquals(LCDEDIT_INIT,field.status);
}

void loop() {
  suite.run();
}
