/*
  UtLcdFields.ino
  Unit tests.

  release : 0.0.0 - 7 April 2011

  ------------------------------------------------------------------------------
  Copyright (c) 2011 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include <Arduino.h>
#include <LiquidCrystal.h>
#include "ArduinoUnit.h"
#include "LcdUi.h"

TestSuite suite;

char DEFC[] PROGMEM = "cac";       // character
char DEFB[] PROGMEM = "b1133";     // byte
char DEFI[] PROGMEM = "i4477";     // integer
char DEFS[] PROGMEM = "saacc";     // string
char DEFT[] PROGMEM = "t07:32:33"; // time
char DEFD[] PROGMEM = "d18/03/50"; // date

PGM_VOID_P DEF[] PROGMEM = {0,DEFC,DEFB,DEFI,DEFS,DEFT,DEFD,0};

char str_i[32];
LcdFields fields;

void setup() {}

test(init) {
  str_i[0] = 0;
  fields.DEF = DEF;
  fields.data = str_i;
  LcdFieldsInit(&fields,true);
  assertEquals(23,fields.length);
  str_i[23] = 0;
  assertEquals("a1144aa07:32:3318/03/50",fields.data);
}

test(edit_field_1_character) {
  str_i[0] = 0;
  fields.DEF = DEF;
  fields.data = str_i;
  LcdFieldsInit(&fields,true);
  str_i[23] = 0;
  LCDKEY key = LCDKEY_SELECT;                             // start edit fields
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_SELECT;                                    // start edit field
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_UP;                                        // character
  LcdFieldsEdit(key,&fields);
  assertEquals("b1144aa07:32:3318/03/50",fields.data);
}

test(edit_field_2_byte) {
  str_i[0] = 0;
  fields.DEF = DEF;
  fields.data = str_i;
  LcdFieldsInit(&fields,true);
  str_i[23] = 0;
  LCDKEY key = LCDKEY_SELECT;                             // start edit fields
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // byte
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_SELECT;                                    // start edit field
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_UP;
  LcdFieldsEdit(key,&fields);
  assertEquals("a2144aa07:32:3318/03/50",fields.data);
}

test(edit_field_field_3_integer) {
  str_i[0] = 0;
  fields.DEF = DEF;
  fields.data = str_i;
  LcdFieldsInit(&fields,true);
  str_i[23] = 0;
  LCDKEY key = LCDKEY_SELECT;                             // start edit fields
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // byte
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // integer
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_SELECT;                                    // start edit field
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_UP;
  LcdFieldsEdit(key,&fields);
  assertEquals("a1154aa07:32:3318/03/50",fields.data);
}

test(edit_field_4_string) {
  str_i[0] = 0;
  fields.DEF = DEF;
  fields.data = str_i;
  LcdFieldsInit(&fields,true);
  str_i[23] = 0;
  LCDKEY key = LCDKEY_SELECT;                             // start edit fields
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // byte
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // integer
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // string
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_SELECT;                                    // start edit field
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_UP;
  LcdFieldsEdit(key,&fields);
  assertEquals("a1144ba07:32:3318/03/50",fields.data);
}

test(edit_field_5_time) {
  str_i[0] = 0;
  fields.DEF = DEF;
  fields.data = str_i;
  LcdFieldsInit(&fields,true);
  str_i[23] = 0;
  LCDKEY key = LCDKEY_SELECT;                             // start edit fields
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // byte
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // integer
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // string
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // time
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_SELECT;                                    // start edit field
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_UP;
  LcdFieldsEdit(key,&fields);
  assertEquals("a1144aa17:32:3318/03/50",fields.data);
}

test(edit_field_6_date) {
  str_i[0] = 0;
  fields.DEF = DEF;
  fields.data = str_i;
  LcdFieldsInit(&fields,true);
  str_i[23] = 0;
  LCDKEY key = LCDKEY_SELECT;                             // start edit fields
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // byte
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // integer
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // string
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // time
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // date
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_SELECT;                                    // start edit field
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_UP;
  LcdFieldsEdit(key,&fields);
  assertEquals("a1144aa07:32:3328/03/50",fields.data);
}

test(navigate) {
  str_i[0] = 0;
  fields.DEF = DEF;
  fields.data = str_i;
  LcdFieldsInit(&fields,true);
  str_i[23] = 0;
  LCDKEY key = LCDKEY_RIGHT;
  LcdFieldsEdit(key,&fields);
  assertEquals(0,fields.cursor);
  key = LCDKEY_SELECT;                                  // start edit fields
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // byte
  LcdFieldsEdit(key,&fields);
  assertEquals(1,fields.cursor);
}

test(next_field) {
  str_i[0] = 0;
  fields.DEF = DEF;
  fields.data = str_i;
  LcdFieldsInit(&fields,true);
  str_i[23] = 0;
  LCDKEY key = LCDKEY_SELECT;                             // start edit fields
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // byte
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_SELECT;                                    // start edit field
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_UP;
  LcdFieldsEdit(key,&fields);
  assertEquals("a2144aa07:32:3318/03/50",fields.data);
}

test(previous_field) {
  str_i[0] = 0;
  fields.DEF = DEF;
  fields.data = str_i;
  LcdFieldsInit(&fields,true);
  str_i[23] = 0;
  LCDKEY key = LCDKEY_SELECT;                             // start edit fields
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_RIGHT;                                     // byte
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_LEFT;                                      // character
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_SELECT;                                    // start edit field
  LcdFieldsEdit(key,&fields);
  key = LCDKEY_UP;
  LcdFieldsEdit(key,&fields);
  assertEquals("b1144aa07:32:3318/03/50",fields.data);
}

void loop() {
  suite.run();
}
