/*
  UTLcdTime.ino
  Unit tests

  release : 0.0.0 - 7 April 2011

  ------------------------------------------------------------------------------
  Copyright (c) 2011 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include "Arduino.h"
#include "ArduinoUnit.h"
#include "LcdUi.h"

TestSuite suite;

void setup() {}

test(refresh) {
  LcdTime dt = {0,1,1,0,0,0};
  LcdTimeInit();
  LcdTimeRefresh(&dt);
  assertEquals(0,dt.second);
}

test(second) {
  LcdTime dt = {0,1,1,0,0,0};
  LcdTimeInit();
  delay(2500);
  LcdTimeRefresh(&dt);
  assertEquals(2,dt.second);
}

test(minute) {
  LcdTime dt = {0,1,1,0,0,59};
  LcdTimeInit();
  delay(1500);
  LcdTimeRefresh(&dt);
  assertEquals(0,dt.second);
  assertEquals(1,dt.minute);
}

test(hour) {
  LcdTime dt = {0,1,1,0,59,59};
  LcdTimeInit();
  delay(1500);
  LcdTimeRefresh(&dt);
  assertEquals(0,dt.second);
  assertEquals(0,dt.minute);
  assertEquals(1,dt.hour);
}

test(day) {
  LcdTime dt = {0,1,1,23,59,59};
  LcdTimeInit();
  delay(1500);
  LcdTimeRefresh(&dt);
  assertEquals(0,dt.second);
  assertEquals(0,dt.minute);
  assertEquals(0,dt.hour);
  assertEquals(2,dt.day);
}

test(january) {
  LcdTime dt = {0,1,31,23,59,59};
  LcdTimeInit();
  delay(1500);
  LcdTimeRefresh(&dt);
  assertEquals(0,dt.second);
  assertEquals(0,dt.minute);
  assertEquals(0,dt.hour);
  assertEquals(1,dt.day);
  assertEquals(2,dt.month);
}

test(february) {
  LcdTime dt = {1,2,28,23,59,59};
  LcdTimeInit();
  delay(1500);
  LcdTimeRefresh(&dt);
  assertEquals(0,dt.second);
  assertEquals(0,dt.minute);
  assertEquals(0,dt.hour);
  assertEquals(1,dt.day);
  assertEquals(3,dt.month);
}

test(february_leap) {
  LcdTime dt = {0,2,29,23,59,59};
  LcdTimeInit();
  delay(1500);
  LcdTimeRefresh(&dt);
  assertEquals(0,dt.second);
  assertEquals(0,dt.minute);
  assertEquals(0,dt.hour);
  assertEquals(1,dt.day);
  assertEquals(3,dt.month);
}

test(year) {
  LcdTime dt = {0,12,31,23,59,59};
  LcdTimeInit();
  delay(1500);
  LcdTimeRefresh(&dt);
  assertEquals(0,dt.second);
  assertEquals(0,dt.minute);
  assertEquals(0,dt.hour);
  assertEquals(1,dt.day);
  assertEquals(1,dt.month);
  assertEquals(1,dt.year);
}

test(weekday_first_valid_date) {
  LcdTime dt = {1,1,1,0,0,0};
  assertEquals(1,LcdTimeWeekday(&dt));
}

test(weekday_2_year) {
  LcdTime dt = {2,1,1,0,0,0};
  assertEquals(2,LcdTimeWeekday(&dt));
}

test(weekday_present) {
  LcdTime dt = {11,4,25,0,0,0};
  assertEquals(1,LcdTimeWeekday(&dt));
}

void loop() {
  suite.run();
}
