/*
  LcdEdit.c

  release : 0.0.1 - 23 Jan 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include "LcdEdit.h"

/*
  Edit character.

  parameter:
  DEF       pointer to input definition = clu : character-lower-upper
  key       key to process
  data      edited character
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/

LCDKEY LcdEditChar_P(PGM_P DEF,LCDKEY key,char* data) {
  switch (key) {
    case LCDKEY_NONE: case LCDKEY_SELECT: key = LCDKEY_NONE; break;
    case LCDKEY_LEFT: case LCDKEY_RIGHT: case LCDKEY_ESC: break;
    case LCDKEY_DOWN:
      --*data;
      key = LCDKEY_NONE;
      break;
    case LCDKEY_UP:
      ++*data;
      key = LCDKEY_NONE;
      break;
  }
  if (*data < pgm_read_byte(DEF + 1)) *data = pgm_read_byte(DEF + 1); // lower
  if (*data > pgm_read_byte(DEF + 2)) *data = pgm_read_byte(DEF + 2); // upper
  return key;
}

/*
  Edit string.

  parameter :
  DEF       pointer to input definition = slu : string-lower string-upper string
  key       key to process
  length    string length
  data      edited string
  x         editing position
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/

LCDKEY LcdEditString_P(PGM_P DEF,uint8_t length,char* x,LCDKEY key,char* data) {
  switch (key) {
    case LCDKEY_NONE: case LCDKEY_SELECT: key = LCDKEY_NONE; break;
    case LCDKEY_ESC: break;
    case LCDKEY_LEFT:
      if (*x > 0) {
        --*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_RIGHT:
      if (*x < length - 1) {
        ++*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_DOWN:
      --data[*x];
      key = LCDKEY_NONE;
      break;
    case LCDKEY_UP:
      ++data[*x];
      key = LCDKEY_NONE;
      break;
  }
  uint8_t i;
  for (i = 0;i < length;i++) {
    if (data[i] < pgm_read_byte(DEF + 1 + i)) data[i] = pgm_read_byte(DEF + 1 + i); // lower
    if (data[i] > pgm_read_byte(DEF + 1 + length + i)) data[i] = pgm_read_byte(DEF + 1 + length + i); // upper
  }
  return key;
}

/*
  Edit byte.

  parameter :
  DEF       pointer to input definition = slu : string-lower string-upper string
  key       key to process
  length    string length
  number    edited byte
  data      edited string
  x         editing position
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
LCDKEY LcdEditByte(uint8_t length,uint8_t lower,uint8_t upper,char* x,LCDKEY key,char* data) {
  uint8_t numberOld = LcdStringStob(data,length);
  switch (key) {
    case LCDKEY_NONE: case LCDKEY_SELECT: key = LCDKEY_NONE; break;
    case LCDKEY_ESC: break;
    case LCDKEY_LEFT:
      if (*x > 0) {
        --*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_RIGHT:
      if (*x < length - 1) {
        ++*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_DOWN:
      --data[*x];
      key = LCDKEY_NONE;
      break;
    case LCDKEY_UP:
      ++data[*x];
      key = LCDKEY_NONE;
      break;
  }
  uint8_t number = LcdStringStob(data,length);
  if (number < lower) number = (numberOld < lower) ? lower : (numberOld > upper) ? upper : numberOld;
  if (number > upper) number = (numberOld > upper) ? upper : (numberOld < lower) ? lower : numberOld;
  LcdStringItos(number,data,length);
  return key;
}

/*
  Edit byte.

  parameter :
  DEF       pointer to input definition = slu : string-lower string-upper string
  key       key to process
  length    string length
  number    edited byte
  data      edited string
  x         editing position
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
LCDKEY LcdEditByte_P(PGM_P DEF,uint8_t length,char* x,LCDKEY key,char* data) {
  uint8_t numberOld = LcdStringStob(data,length);
  switch (key) {
    case LCDKEY_NONE: case LCDKEY_SELECT: key = LCDKEY_NONE; break;
    case LCDKEY_ESC: break;
    case LCDKEY_LEFT:
      if (*x > 0) {
        --*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_RIGHT:
      if (*x < length - 1) {
        ++*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_DOWN:
      --data[*x];
      key = LCDKEY_NONE;
      break;
    case LCDKEY_UP:
      ++data[*x];
      key = LCDKEY_NONE;
      break;
  }
  uint8_t number = LcdStringStob(data,length);
  uint8_t lower = LcdStringStob_P(DEF + 1,length);
  uint8_t upper = LcdStringStob_P(DEF + 1 + length,length);
  if (number < lower) number = (numberOld < lower) ? lower : (numberOld > upper) ? upper : numberOld;
  if (number > upper) number = (numberOld > upper) ? upper : (numberOld < lower) ? lower : numberOld;
  LcdStringItos(number,data,length);
  return key;
}

/*
  Edit integer.

  parameter :
  DEF       pointer to input definition = slu : string-lower string-upper string
  key       key to process
  length    string length
  number    edited integer
  data      edited string
  x         editing position
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
LCDKEY LcdEditInt_P(PGM_P DEF,uint8_t length,char* x,LCDKEY key,char* data) {
  int16_t numberOld = LcdStringStoi(data,length);
  switch (key) {
    case LCDKEY_NONE: case LCDKEY_SELECT: key = LCDKEY_NONE; break;
    case LCDKEY_ESC: break;
    case LCDKEY_LEFT:
      if (*x > 0) {
        --*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_RIGHT:
      if (*x < length - 1) {
        ++*x;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_DOWN:
      --data[*x];
      key = LCDKEY_NONE;
      break;
    case LCDKEY_UP:
      ++data[*x];
      key = LCDKEY_NONE;
      break;
  }
  int16_t number = LcdStringStoi(data,length);
  int16_t lower = LcdStringStoi_P(DEF + 1,length);
  int16_t upper = LcdStringStoi_P(DEF + 1 + length,length);
  if (number < lower) number = (numberOld < lower) ? lower : (numberOld > upper) ? upper : numberOld;
  if (number > upper) number = (numberOld > upper) ? upper : (numberOld < lower) ? lower : numberOld;
  LcdStringItos(number,data,length);
  return key;
}
