/*
  LcdEditTime.c

  release : 0.0.1 - 23 Jan 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include "LcdEditTime.h"

/*
  Edit date.

  parameter :
  dt        date and time
  key       key to process
  data      edited date time
  x         editing position
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
LCDKEY LcdEditTimeDate(char* x,LCDKEY key,char* data) {
  if (key == LCDKEY_NONE) return LCDKEY_NONE;
  uint8_t dx;
  LcdTime dt;
  switch (*x) {
    case 0: case 1:
      dx = *x;
      key = LcdEditByte(2,1,31,&dx,key,&data[0]);
      *x = dx;
      break;
    case 3: case 4:
      dx = *x - 3;
      key = LcdEditByte(2,1,12,&dx,key,&data[3]);
      *x = dx + 3;
      break;
    case 6: case 7:
      dx = *x - 6;
      key = LcdEditByte(2,0,99,&dx,key,&data[6]);
      *x = dx + 6;
      break;
  }
  LcdStringStod(data,&dt);
  if (dt.day > LcdTimeDaymax(&dt)) {
    dt.day = LcdTimeDaymax(&dt);
    LcdStringDtos(&dt,data);
  }
  switch (key) {
    case LCDKEY_ESC: break;
    case LCDKEY_NONE: case LCDKEY_SELECT: case LCDKEY_DOWN: case LCDKEY_UP: key = LCDKEY_NONE; break;
    case LCDKEY_LEFT:
      if (*x - dx > 0) {
        *x -= 2;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_RIGHT:
      if (*x - dx < 6) {
        *x += 2;
        key = LCDKEY_NONE;
      }
      break;
  }
  return key;
}

/*
  Edit time.

  parameter :
  dt        date and time
  key       key to process
  data      edited time
  x         editing position
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
LCDKEY LcdEditTimeTime(char* x,LCDKEY key,char* data) {
  if (key == LCDKEY_NONE) return LCDKEY_NONE;
  uint8_t dx;
  LcdTime dt;
  LcdStringStot(data,&dt);
  LcdStringTtos(&dt,data);
  switch (*x) {
    case 0: case 1:
      dx = *x;
      key = LcdEditByte(2,0,23,&dx,key,&data[0]);
      *x = dx;
      break;
    case 3: case 4:
      dx = *x - 3;
      key = LcdEditByte(2,0,59,&dx,key,&data[3]);
      *x = dx + 3;
      break;
    case 6: case 7:
      dx = *x - 6;
      key = LcdEditByte(2,0,59,&dx,key,&data[6]);
      *x = dx + 6;
      break;
  }
  switch (key) {
    case LCDKEY_ESC: break;
    case LCDKEY_NONE: case LCDKEY_SELECT: case LCDKEY_DOWN: case LCDKEY_UP: key = LCDKEY_NONE; break;
    case LCDKEY_LEFT:
      if (*x - dx > 0) {
        *x -= 2;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_RIGHT:
      if (*x - dx < 6) {
        *x += 2;
        key = LCDKEY_NONE;
      }
      break;
  }
  return key;
}
