/*
  LcdField.c

  release : 0.0.1 - 23 Jan 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include "LcdField.h"

/*
  Edit field.

  parameter :
  field     field data
  key       key to process
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
LCDKEY LcdFieldEdit(LCDKEY key,LcdField* field) {
  switch (key) {
    case LCDKEY_NONE: break;
    case LCDKEY_SELECT:
      if (field->status == LCDEDIT_INIT) {
        field->status = LCDEDIT_EDIT;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_ESC:
      if (field->status == LCDEDIT_EDIT || field->status == LCDEDIT_CHANGED) {
        field->status = LCDEDIT_INIT;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_EXIT: if (field->status == LCDEDIT_CHANGED) field->status = LCDEDIT_INIT; break;
    case LCDKEY_UP: case LCDKEY_DOWN:
      if (field->status == LCDEDIT_EDIT) field->status = LCDEDIT_CHANGED;
    case LCDKEY_LEFT: case LCDKEY_RIGHT:
      if (field->status == LCDEDIT_EDIT || field->status == LCDEDIT_CHANGED) {
        switch (pgm_read_byte(field->DEF)) {
          case 'b': key = LcdEditByte_P(field->DEF,field->length,&field->x,key,field->data); break;
          case 'c': key = LcdEditChar_P(field->DEF,key,field->data); break;
          case 'd': key = LcdEditTimeDate(&field->x,key,field->data); break;
          case 'i': key = LcdEditInt_P(field->DEF,field->length,&field->x,key,field->data); break;
          case 's': key = LcdEditString_P(field->DEF,field->length,&field->x,key,field->data); break;
          case 't': key = LcdEditTimeTime(&field->x,key,field->data); break;
        }
      }
      break;
  }
  return key;
}

/*
  Initialise field data.
  - copy field data pointers
  - get field length
  - optionally initialise string data.

  parameter :
  field   field data
  init    initialise data
*/
void LcdFieldInit(LcdField* field,boolean init) {
  field->x = 0;
  field->length = LcdFieldLength(field->DEF);
  if (pgm_read_byte(field->DEF) == 'o') {
    field->status = LCDEDIT_NONE;
    if (init) memset(field->data,'-',field->length);
  } else {
    field->status = LCDEDIT_INIT;
    if (init) memcpy_P(field->data,field->DEF + 1,field->length);
  }
}

/*
  Get field length.

  paramete :s
  DEF       pointer to field definition
  return    field length
*/
uint8_t LcdFieldLength(PGM_VOID_P DEF) {
  switch(pgm_read_byte(DEF)) {
    case 'b': case 'i': case 's': return (strlen_P(DEF) - 1) >> 1;
    case 'c': return 1;
    case 'd': case 't': return 8;
    case 'o': return strlen_P(DEF) - 1;
  }
}
