/*
  LcdField.h
  Process a field.
  Various field types exist (character, byte, integer, string, date, time,...).
  In this context (input/editing/output) they are all handled as strings which
  are stored in data memory (RAM) as one contigous string.
  The field definitions however are stored in program memory (FLASH).
  Editing is processed by the relevant LcdEdit and LcdEditTime routines.

  release : 0.0.1 - 23 Jan 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#ifdef __cplusplus
extern "C" {
#endif

#ifndef LcdField_h
#define LcdField_h
#include <Arduino.h>
#include "LcdEdit.h"
#include "LcdEditTime.h"
#include "LcdKey.h"
#include "LcdString.h"
typedef struct {
  PGM_VOID_P  DEF;
  uint8_t     length;
  char*       data;
  uint8_t     x;
  LCDEDIT     status;
  uint8_t      cursor;
} LcdField;
#endif
LCDKEY    LcdFieldEdit(LCDKEY key,LcdField* field);
void      LcdFieldInit(LcdField* field,boolean init);
uint8_t   LcdFieldLength(PGM_VOID_P DEF);

#ifdef __cplusplus
}
#endif
