/*
  LcdFields.c

  release : 0.0.1 - 23 Jan 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
*/

#include "LcdFields.h"

/*
  Edit Fields.

  parameter :
  key       key to process
  fields    fields data
  return    key, if not consumed
            LCDKEY_NONE otherwise
*/
LCDKEY LcdFieldsEdit(LCDKEY key,LcdFields* fields) {
  int8_t i = 0;
  switch (key) {
    case LCDKEY_NONE: break;
    case LCDKEY_SELECT:
      if (fields->status == LCDEDIT_INIT) {
        fields->status = LCDEDIT_EDIT;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_ESC:
      if (fields->field.status == LCDEDIT_INIT) {
        fields->status = LCDEDIT_INIT;
        key = LCDKEY_NONE;
      }
      break;
    case LCDKEY_EXIT: if (fields->status != LCDEDIT_NONE) fields->status = LCDEDIT_INIT; break;
    case LCDKEY_DOWN: if (fields->status == LCDEDIT_EDIT && fields->field.status == LCDEDIT_INIT) i = 127; break;
    case LCDKEY_UP: if (fields->status == LCDEDIT_EDIT && fields->field.status == LCDEDIT_INIT) i = -128; break;
    case LCDKEY_LEFT: if (fields->status == LCDEDIT_EDIT && fields->field.status == LCDEDIT_INIT) i = -1 ;break;
    case LCDKEY_RIGHT: if (fields->status == LCDEDIT_EDIT && fields->field.status == LCDEDIT_INIT) i = 1; break;
  }
  if (i) {
    boolean found = false;
    PGM_VOID_P DEF = fields->DEF;
    char* data = fields->field.data;
    while (i < 0  && !found) {
      fields->DEF -= 2;
      if ((fields->field.DEF = pgm_read_word(fields->DEF))) {
        ++i;
        fields->field.data -= LcdFieldLength(fields->field.DEF);
        LcdFieldInit(&fields->field,false);
        if (fields->field.status == LCDEDIT_INIT) found = true;
      } else i = 0;
    }
    while (i > 0 && !found) {
      fields->DEF += 2;
      if ((fields->field.DEF = pgm_read_word(fields->DEF))) {
        --i;
        fields->field.data += fields->field.length;
        LcdFieldInit(&fields->field,false);
        if (fields->field.status == LCDEDIT_INIT) found = true;
      } else i = 0;
    }
    if (!found) {
      fields->DEF = DEF;
      fields->field.DEF = pgm_read_word(fields->DEF);
      fields->field.data = data;
      LcdFieldInit(&fields->field,false);
    }
    key = LCDKEY_NONE;
  }
  key = LcdFieldEdit(key,&fields->field);
  fields->cursor = fields->field.data - fields->data + fields->field.x;
  return key;
}

/*
  Initialise Fields.

  parameter :
  fields    fields data
  init      initialise data
*/
void LcdFieldsInit(LcdFields* fields,boolean init) {
  fields->length = LcdFieldsLength(fields->DEF);
  fields->status = LCDEDIT_NONE;
  if (!fields->length) return;
  fields->DEF += 2;
  fields->field.data = fields->data;
  PGM_VOID_P DEF = fields->DEF;
  while ((fields->field.DEF = pgm_read_word(DEF))) {
    LcdFieldInit(&fields->field,init);
    if (fields->field.status == LCDEDIT_INIT) fields->status = LCDEDIT_INIT;
    fields->field.data += fields->field.length;
    DEF += 2;
  }
  if (fields->status == LCDEDIT_INIT) {
    fields->field.data = fields->data;
    boolean found = false;
    while (!found) {
      fields->field.DEF = pgm_read_word(fields->DEF);
      LcdFieldInit(&fields->field,false);
      if (fields->field.status == LCDEDIT_NONE) {
        fields->DEF += 2;
        fields->field.data += fields->field.length;
      } else found = true;
    }
  }
}

/*
  Get fields length.

  parameter :
  DEF       pointer to fields definition
  return    fields length
*/
uint8_t LcdFieldsLength(PGM_VOID_P DEF) {
  if (!DEF) return 0;
  DEF += 2;
  uint8_t length = 0;
  PGM_VOID_P PTR;
  while ((PTR = pgm_read_word(DEF))) {
    length += LcdFieldLength(PTR);
    DEF += 2;
  }
  return length;
}
