/*
  LcdString.c

  release : 0.0.1 - 23 Jan 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
 */

#include "LcdString.h"

/*
  Date to string.
  Fixed format result : "DD/MM/YY"

  parameter :
  dt      date/time
  data    string
*/
void LcdStringDtos(LcdTime* dt,char* data) {
  LcdStringItos(dt->day,&data[0],2);
  data[2] = '/';
  LcdStringItos(dt->month,&data[3],2);
  data[5] = '/';
  LcdStringItos(dt->year,&data[6],2);
}

/*
  Integer to string.

  parameter :
  number    number to be converted
  data      string data
  length    string length
*/
void LcdStringItos(uint16_t number,char* data,uint8_t length) {
  uint8_t i = length;
  while (i) {
    data[--i] = number % 10 + 48;
    number = number / 10;
  }
}

/*
  Time to string.
  Fixed format result : "hh:mm:ss"

  parameter :
  dt      date/time
  data    string
*/
void LcdStringTtos(LcdTime* dt,char* data) {
  LcdStringItos(dt->hour,&data[0],2);
  data[2] = ':';
  LcdStringItos(dt->minute,&data[3],2);
  data[5] = ':';
  LcdStringItos(dt->second,&data[6],2);
}

/*
  String to byte.

  parameter :
  data      string
  length    string length
  return    byte
*/
uint8_t LcdStringStob(char* data,uint8_t length) {
  uint8_t number = 0;
  uint8_t i;
  for (i = 0;i < length;i++) number = number * 10 + data[i] - 48;
  return number;
}

/*
  String to byte (from flash).

  parameter :
  DATA      PROGMEM data
  length    data length
  return    byte
*/
uint8_t LcdStringStob_P(PGM_P DATA,uint8_t length) {
  uint8_t number = 0;
  uint8_t i;
  for (i = 0;i < length;i++) number = number * 10 + pgm_read_byte(DATA + i) - 48;
  return number;
}

/*
  String to date.

  parameter :
  data    string
  dt      date/time
*/
void LcdStringStod(char* data,LcdTime* dt) {
  dt->day = LcdStringStob(&data[0],2);
  dt->month = LcdStringStob(&data[3],2);
  dt->year = LcdStringStob(&data[6],2);
}

/*
  String to date (from flash).

  parameter :
  DATA    PROGMEM data
  dt      date/time
*/
void LcdStringStod_P(PGM_P DATA,LcdTime* dt) {
  dt->day = LcdStringStob_P(DATA,2);
  dt->month = LcdStringStob_P(DATA + 3,2);
  dt->year = LcdStringStob_P(DATA + 6,2);
}

/*
  String to integer.

  parameter :
  data      string
  length    string length
  return    integer
*/
int16_t LcdStringStoi(char* data,uint8_t length) {
  int16_t number = 0;
  uint8_t i;
  for (i = 0;i < length;i++) number = number * 10 + data[i] - 48;
  return number;
}

/*
  String to integer (from flash).

  parameter :
  DATA      PROGMEM data
  length    data length
  return    integer
*/
int16_t LcdStringStoi_P(PGM_P DATA,uint8_t length) {
  int16_t number = 0;
  uint8_t i;
  for (i = 0;i < length;i++) number = number * 10 + pgm_read_byte(DATA + i) - 48;
  return number;
}

/*
  String to time.

  parameter :
  data    string
  dt      date/time
*/
void LcdStringStot(char* data,LcdTime* dt) {
  dt->hour = LcdStringStob(&data[0],2);
  dt->minute = LcdStringStob(&data[3],2);
  dt->second = LcdStringStob(&data[6],2);
}

/*
  String to time (from flash).

  parameter :
  DATA    PROGMEM data
  dt      date/time
*/
void LcdStringStot_P(PGM_P DATA,LcdTime* dt) {
  dt->hour = LcdStringStob_P(DATA,2);
  dt->minute = LcdStringStob_P(DATA + 3,2);
  dt->second = LcdStringStob_P(DATA + 6,2);
}
